import fs from 'fs';
import path from 'path';

// Define an enum for message file names
export enum MessageFileName {
    RdramaPreviousMessage = 'rdrama_PreviousMessage.txt',
    RdramaShouldntNotify = 'rdrama_ShouldntNotify.txt',
    RedditMessages = 'reddit_messages.txt',
    RdramaMessages = 'rdrama_messages.txt',
}

export class MessageService {
    /**
     * Loads messages from a specified text file based on the enum, splitting by a specific delimiter.
     * Each message is separated by '---END---' in the text file.
     * 
     * @param {MessageFileName} fileName - The enum value representing the file containing the messages.
     * @returns {string[] | undefined} An array of messages, or undefined if there was an error loading the messages.
     */
    private static loadMessages(fileName: MessageFileName): string[] | undefined {
        try {
            const messagesPath = path.join(__dirname, '..', 'messages', fileName);
            return fs.readFileSync(messagesPath, 'utf-8').split('---END---').filter(line => line.trim());
        } catch (error) {
            console.error(`Failed to load messages from ${fileName}:`, error);
        }
    }

    /**
     * Selects a random message from the loaded messages and replaces placeholders within it.
     * 
     * @param {MessageFileName} fileName - The enum value representing the file containing the messages to load.
     * @param {Object} placeholders - A mapping of placeholder names to their replacement values.
     * @returns {string | undefined} A formatted message with placeholders replaced, or undefined if messages couldn't be loaded.
     */
    public static getRandomMessage(fileName: MessageFileName, placeholders: { [key: string]: string }): string | undefined {
        const messages = this.loadMessages(fileName);
        if (!messages) return undefined;
        const message = messages[Math.floor(Math.random() * messages.length)];
        return this.replacePlaceholders(message, placeholders);
    }

    /**
     * Replaces placeholders in a message with values from a provided mapping.
     * 
     * @param {string} message - The message containing placeholders.
     * @param {Object} placeholders - A mapping of placeholder names to their replacement values.
     * @returns {string} The message with placeholders replaced by actual values.
     */
    private static replacePlaceholders(message: string, placeholders: { [key: string]: string }): string {
        return Object.keys(placeholders).reduce((acc, key) => {
            const regex = new RegExp(`{${key}}`, 'g');
            return acc.replace(regex, placeholders[key]);
        }, message);
    }
}
